/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/
#include <stdlib.h>
#include "ER-EPD029-1R.h"
#include "Debug.h"
#include "GUI_Paint.h"

const unsigned char lut[] = {
0xAA,	0x99,	0x10,	0x00,	0x00,	0x00,	0x00,	0x55,	0x99,	0x80,	0x00,	0x00,	0x00,	0x00,	0x8A,	0xA8,
0x9B,	0x00,	0x00,	0x00,	0x00,	0x8A,	0xA8,	0x9B,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x0F,	0x0F,	0x0F,	0x0F,	0x02,	0x14,	0x14,	0x14,	0x14,	0x06,	0x14,	0x14,	0x0C,
0x82,	0x08,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,									

/*
0xA5,	0x89,	0x10,	0x00,	0x00,	0x00,	0x00,	0xA5,	0x19,	0x80,	0x00,	0x00,	0x00,	0x00,	0xA5,	0xA9,
0x9B,	0x00,	0x00,	0x00,	0x00,	0xA5,	0xA9,	0x9B,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x0F,	0x0F,	0x0F,	0x0F,	0x02,	0x10,	0x10,	0x0A,	0x0A,	0x03,	0x08,	0x08,	0x09,
0x43,	0x07,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,										
*/	
};



/******************************************************************************
function :	Software reset
parameter:
******************************************************************************/
void EPD2IN9::EPD_Reset(void)
{
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
    EPD_RST_0;
    Dev->Dev_Delay_ms(200);
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
}

/******************************************************************************
function :	send command
parameter:
    Reg : Command register
******************************************************************************/
void EPD2IN9::EPD_SendCommand(UBYTE Reg)
{
    EPD_DC_0;
    EPD_CS_0;
    SPI_Write_Byte(Reg);
    EPD_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
void EPD2IN9::EPD_SendData(UBYTE Data)
{
    EPD_DC_1;
    EPD_CS_0;
    SPI_Write_Byte(Data);
    EPD_CS_1;
}

/******************************************************************************
function :	Wait until the busy_pin goes LOW
parameter:
******************************************************************************/
void EPD2IN9::EPD_WaitUntilIdle(void)
{
    while(EPD_BUSY_RD == HIGH) {      //LOW: idle, HIGH: busy
        Dev->Dev_Delay_ms(100);
    }
}

/******************************************************************************
function :	Setting the display window
parameter:
******************************************************************************/
void EPD2IN9::EPD_SetWindows(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend)
{
    EPD_SendCommand(SET_RAM_X_ADDRESS_START_END_POSITION);
    EPD_SendData((Xstart >> 3) & 0xFF);
    EPD_SendData((Xend >> 3) & 0xFF);

    EPD_SendCommand(SET_RAM_Y_ADDRESS_START_END_POSITION);
    EPD_SendData(Ystart & 0xFF);
    EPD_SendData((Ystart >> 8) & 0xFF);
    EPD_SendData(Yend & 0xFF);
    EPD_SendData((Yend >> 8) & 0xFF);
}

/******************************************************************************
function :	Set cursor
parameter:
******************************************************************************/
void EPD2IN9::EPD_SetCursor(UWORD Xstart, UWORD Ystart)
{
    EPD_SendCommand(SET_RAM_X_ADDRESS_COUNTER);
    EPD_SendData((Xstart >> 3) & 0xFF);

    EPD_SendCommand(SET_RAM_Y_ADDRESS_COUNTER);
    EPD_SendData(Ystart & 0xFF);
    EPD_SendData((Ystart >> 8) & 0xFF);
    
    EPD_WaitUntilIdle();
}

/******************************************************************************
function :	Turn On Display
parameter:
******************************************************************************/
void EPD2IN9::EPD_TurnOnDisplay(void)
{
    EPD_SendCommand(DISPLAY_UPDATE_CONTROL_2);
    EPD_SendData(0xC7);
    EPD_SendCommand(MASTER_ACTIVATION);
    EPD_WaitUntilIdle();
}

/**
 *  @brief: set the look-up table register
 */
void EPD2IN9::EPD_SetLut() {
    EPD_SendCommand(WRITE_LUT_REGISTER);
    /* the length of look-up table is 30 bytes */
    for (int i = 0; i < 70; i++) {
    EPD_SendData(lut[i]);
    }
}



/******************************************************************************
function :	Initialize the e-Paper register
parameter:
******************************************************************************/
UBYTE EPD2IN9::EPD_Init()
{
    Dev->System_Init();

    EPD_Reset();
      /* 
    EPD_SendCommand(0x74);  //Set Analog Block Control
    EPD_SendData(0x54);
    EPD_SendCommand(0x7E);  //Set Digital Block Control
    EPD_SendData(0x3B);
    EPD_SendCommand(0x2B);  // Reduce glitch under ACVCOM	
    EPD_SendData(0x04);           
    EPD_SendData(0x63);
 

    EPD_SendCommand(BOOSTER_SOFT_START_CONTROL);		  // Booster Soft start Control
    EPD_SendData(0x8B);		   
    EPD_SendData(0x9C);		   
    EPD_SendData(0x96);			   
    EPD_SendData(0x0F);
    
        
    EPD_SendCommand(DRIVER_OUTPUT_CONTROL);  // Driver Output control
    EPD_SendData(0x27);  
    EPD_SendData(0x01);
    EPD_SendData(0x00);     
        
    EPD_SendCommand(DATA_ENTRY_MODE_SETTING);  // Data Entry mode setting
    EPD_SendData(0x03);         
    EPD_SendCommand(SET_RAM_X_ADDRESS_START_END_POSITION); //Set RAM X - address Start / End position
    EPD_SendData(0x00); // RAM x address start at 0
    EPD_SendData(0x0F); //RAM x address end at 0Fh(15+1)*8->128
    EPD_SendCommand(SET_RAM_Y_ADDRESS_START_END_POSITION); //Set Ram Y- address Start / End position
    EPD_SendData(0x27); // RAM y address start 
    EPD_SendData(0x01);
    EPD_SendData(0x00); // RAM y address end 
    EPD_SendData(0x00);
        
        
    EPD_SendCommand(BORDER_WAVEFORM_CONTROL); // Border Waveform Control
    EPD_SendData(0x01); // HIZ

    EPD_SendCommand(DISPLAY_UPDATE_CONTROL_1);
    EPD_SendData(0x08);//Normal

        
 //       unsigned char temp1,temp2;	
    EPD_SendCommand(0x18);//Temperature Sensor Control
    EPD_SendData(0x80);  //Internal temperature sensor
    EPD_SendCommand(DISPLAY_UPDATE_CONTROL_2);//Display UpdateControl 2
    EPD_SendData(0xB1);	//Load Temperature and waveform setting.
    EPD_SendCommand(MASTER_ACTIVATION); //Master Activation
    EPD_WaitUntilIdle();*/	     


    EPD_SendCommand(0x74);
    EPD_SendData(0x54);
    EPD_SendCommand(0x75);
    EPD_SendData(0x3b);
    EPD_SendCommand(0x01);		// Set MUX as 296
    EPD_SendData(0x27);
    EPD_SendData(0x01);
    EPD_SendData(0x00);
    EPD_SendCommand(0x3A);		// Set 100Hz
    EPD_SendData(0x35);         // Set 130Hz
    EPD_SendCommand(0x3B);		// Set 100Hz
    EPD_SendData(0x04);         // Set 130Hz
    EPD_SendCommand(0x11);		// data enter mode
    EPD_SendData(0x03);
    EPD_SendCommand(0x44);		// set RAM x address start/end, in page 36
    EPD_SendData(0x00);		// RAM x address start at 00h;
    EPD_SendData(0x0f);		// RAM x address end at 0fh(15+1)*8->128 
    EPD_SendCommand(0x45);		// set RAM y address start/end, in page 37
    EPD_SendData(0x00);		// RAM y address start at 127h;
    EPD_SendData(0x00);			
    EPD_SendData(0x27);		// RAM y address end at 00h;
    EPD_SendData(0x01);		
    EPD_SendCommand(0x04);		// set VSH,VSL value
    EPD_SendData(0x41);		// 	    2D9  15v
    EPD_SendData(0xa8);		//	    2D9   5v 
    EPD_SendData(0x32);		//	    2D9  -15v
    EPD_SendCommand(0x2C);           // vcom
    EPD_SendData(0x68);           //-2.6V
    EPD_SendCommand(0x3C);		// board
    EPD_SendData(0x33);		//GS1-->GS1
 
    
    EPD_SendCommand(DISPLAY_UPDATE_CONTROL_1);
    EPD_SendData(0x08);//Inverse BW RAM content
             
    EPD_SetLut();	
    return 0;
}

/******************************************************************************
function :	Clear screen
parameter:
******************************************************************************/
void EPD2IN9::EPD_Clear(void) 
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    EPD_SetWindows(0, 0, EPD_WIDTH, EPD_HEIGHT);
    for (UWORD j = 0; j < Height; j++) {
        EPD_SetCursor(0, j);
        EPD_SendCommand(WRITE_RAM_BW);
        for (UWORD i = 0; i < Width; i++) {
            // SendData(color);
            EPD_SendData(0X00);
        }
    } 
    for (UWORD j = 0; j < Height; j++) {
        EPD_SetCursor(0, j);
        EPD_SendCommand(WRITE_RAM_RED);
        for (UWORD i = 0; i < Width; i++) {
            // SendData(color);
            EPD_SendData(0X00);
        }
    }    
    
    
   // EPD_TurnOnDisplay();
}

/******************************************************************************
function :	Sends the image buffer in RAM to e-Paper and displays
parameter:
******************************************************************************/
void EPD2IN9::EPD_Display(void)
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;

    EPD_SetWindows(0, 0, EPD_WIDTH, EPD_HEIGHT);
    for (UWORD j = 0; j < Height; j++) {
        EPD_SetCursor(0, j);
        EPD_SendCommand(WRITE_RAM_BW);
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width));
        }
    }
    for (UWORD j = 0; j < Height; j++) {
        EPD_SetCursor(0, j);
        EPD_SendCommand(WRITE_RAM_RED);
        for (UWORD i = 0; i < Width; i++) {
        EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width + Paint_Image.Image_Offset));
        }
    } 
    
    EPD_TurnOnDisplay();
}

/******************************************************************************
function :	Enter sleep mode
parameter:
******************************************************************************/
void EPD2IN9::EPD_Sleep(void) 
{
    EPD_SendCommand(DEEP_SLEEP_MODE);
    EPD_SendData(0x01);
    EPD_WaitUntilIdle();
}
